package File::DXF::Csv;

=head1 NAME

File::DXF::Csv - Comma separated value files

=head1 SYNOPSIS

Simple CSV writing

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Exporter;
use vars qw /@ISA @EXPORT_OK/;

@ISA = qw /Exporter/;

@EXPORT_OK = qw /array2csv/;

=pod

Create a Csv object like so:

  $cvs = new File::DXF::Csv;

=cut

sub new
{
    my $class = shift;
    $class = ref $class || $class;
    my $self;

    $self = [];

    bless $self, $class;
    return $self;
}

=pod

Add a line of data to the table:

  $cvs->Push (1, 1, 2, 3, 5, 8, 13, 21, 34, 55);

=cut

sub Push
{
    my $self = shift;
    push @{$self}, array2csv (@_);
}

=pod

Output the table as a CSV file like so:

  $cvs->Write ("my-filename.csv");

Note that that line-endings are the system default and that all output is
assumed to be UTF-8.

Warning: Excel is a pile of crap and can't open UTF-8 encoded
CSV files.  If you have non-ascii characters in your data, you
need to use the Excel 'Data import wizard'.

=cut

sub Write
{
    my $self = shift;
    my $filename = shift;

    open FILE, ">:encoding(utf8)", $filename
         or die "cannot write-open $filename";
    print FILE join ("\n", @{$self});
    close FILE;
}

=pod

=head1 Utility methods

Take an array and turn it into a line of a CSV file (not including final
carriage return).

    $line = array2csv (@array);

=cut

sub array2csv
{
    my @line = @_;
    @line = map (_escape_for_csv ($_), @line);
    return (join ",", @line);
}

=pod

" is escaped as "".

Carriage returns are replaced by whitespace.

Leading and trailing spaces are removed.

Items containing " or , are quoted with double quotes.

=cut

sub _escape_for_csv
{
    my $item = shift;
    $item =~ s/"/""/g;
    $item =~ s/[\r\n\l]+/ /g;
    $item =~ s/(^ +| +$)//g;
    $item = "\"$item\"" if $item =~ /[",]/;
    return $item;
}

1;
