package File::DXF::ENTITIES;

=head1 NAME

File::DXF::ENTITIES - Handle AutoCAD DXF ENTITIES list

=head1 SYNOPSIS

ENTITIES is one section of a DXF file, this class inherits from L<File::DXF::Section>

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use File::DXF::Util 'dxf';

use File::DXF::Section 'next_item';
use vars qw /@ISA/;
@ISA = qw /File::DXF::Section/;

=pod

Try and find some structure in the raw data (which needs to be in two column
format as produced by read_DXF().

  $entities->Process ($data);

..or:

  $entities->Process ([[0, 'POINT'],
                       [10, 1.23],
                       [20, 4.56],
                         ...
                       [0, 'TEXT'],
                       [10, 7.89],
                         ...
                       [50, 90]]);

$entities becomes a list of all the entities, each of which are a hash with
'type' entry which is a string representing type and _data consisting of
unprocessed two-column data for that entity.

=cut

sub Process
{
    my $self = shift;
    my $data = shift;
    my $entity_type;
    my $chunk = [];

    while (@{$data})
    {
        my ($code, $string) = next_item ($data);

        if ($code == 0)
        {
            if ($entity_type)
            {
                my $struct = { type => $entity_type,
                              _data => $chunk };
                push @{$self}, $struct;
            }
            $chunk = [];
            $entity_type = $string;
            next;
        }

        push @{$chunk}, [$code, $string];
    }
    if ($entity_type)
    {
        my $struct = { type => $entity_type,
                      _data => $chunk };
        push @{$self}, $struct;
    }
}

=pod

Return a list of entities that match one or more 'types'

  @report = $entities->Report (('TEXT', 'POINT'));

These are blessed into appropriate classes (such as File::DXF::ENTITIES::TEXT).

=cut

sub Report
{
    my $self = shift;
    my @types = @_;
    my @out;

    for my $entity (@{$self})
    {
        next unless (grep /^$entity->{type}$/, @types );
        my $type = $entity->{type};
        eval "use File::DXF::ENTITIES::$type";
        next if $@;
        my $item = eval "new File::DXF::ENTITIES::$type";
        $item->Process ($entity->{_data});
        push @out, $item;
    }
    return @out;
}

=pod

Generate DXF formatted data, just for this section:

  $text = $entities->DXF;

=cut

sub DXF
{
    my $self = shift;
    my $text;
    $text .= dxf (0, 'SECTION');
    $text .= dxf (2, 'ENTITIES');

    for my $entity (@{$self})
    {
        my $type = $entity->{type};
        $text .= dxf (0, $type);
        for my $item (@{$entity->{_data}})
        {
            next if $item->[0] == 5;
            next if $item->[0] == 100;
            next if $item->[0] == 330;
            next if $item->[0] >= 1000;
            $text .= dxf (@{$item});
        }
    }

    $text .= dxf (0, 'ENDSEC');
    return $text;
}

=pod

Find all polyline polyface meshes, and extract them in a simplified format

  @meshes = $entities->Extract_Meshes;

Each mesh is a list of points in space-separated '1.23 4.56 7.89' format,
followed by a list of triangle/quad faces in space-separated '12 34 56 78'
format.

=cut

sub Extract_Meshes
{
    my $self = shift;
    my @report = $self->Report (('POLYLINE', 'VERTEX', 'SEQEND'));

    my $state = 'INIT';
    my $id_node;
    my $id_triangle;
    my @nodes;
    my @triangles;
    my @meshes;

    for my $item (@report)
    {
        if ($item->Type eq 'POLYLINE' and $item->Triangular)
        {
            print STDERR "found triangular mesh polyline\n";
            print STDERR "nodes: ". $item->Nodes ."\n";
            print STDERR "triangles: ". $item->Triangles ."\n";
            $state = 'POINTS';
            $id_node = 1;
            $id_triangle = 1;
            @nodes = ();
            @triangles = ();
        }

        if ($state eq 'POINTS' and $item->Type eq 'VERTEX' and $item->Point)
        {
            # STAAD has a maximum line length limit
            $item->{x} = sprintf ("%.6f", $item->{x});
            $item->{y} = sprintf ("%.6f", $item->{y});
            $item->{z} = sprintf ("%.6f", $item->{z});

            push @nodes, join (' ', $item->{x}, $item->{y}, $item->{z});
            $id_node++;
        }

        if ($state eq 'POINTS' and $item->Type eq 'VERTEX' and $item->Triangle)
        {
    	$item->{a} = 0 - $item->{a} if ($item->{a} < 0);
    	$item->{b} = 0 - $item->{b} if ($item->{b} < 0);
    	$item->{c} = 0 - $item->{c} if ($item->{c} < 0);

            # some triangles have four sides
            if (defined $item->{d})
            {
    	    $item->{d} = 0 - $item->{d} if ($item->{d} < 0);

                push @triangles, join (' ', $item->{a} -1, $item->{b} -1, $item->{c} -1, $item->{d} -1);
            }
            else
            {
                push @triangles, join (' ', $item->{a} -1, $item->{b} -1, $item->{c} -1);
            }

            $id_triangle++;
        }

        if ($item->Type eq 'SEQEND' and $state eq 'POINTS')
        {
            push @meshes, [[@nodes], [@triangles]] if (scalar @nodes and scalar @triangles);
            $state = 'INIT';
        }
    }
    return @meshes;
}

1;

