package File::DXF::Math;

=head1 NAME

File::DXF::Math - Some vector math

=head1 SYNOPSIS

Utility functions

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Math::Trig;
use Exporter;
use vars qw /@ISA @EXPORT_OK/;

@ISA = qw /Exporter/;

@EXPORT_OK = qw /cog_3d average_3d add_3d scale_3d area_3d green_theorem distance_3d normalise_3d x_product_3d dot_product_3d magnitude_3d subtract_3d triangle_normal line_plane_intersection angle_vectors_3d PI angle_triangle point_is_in_triangle flatten_triangle rotate_3d/;

# finds centre of gravity of the volume between a 3d triangle and the XY plane.

sub cog_3d
{
    my @P = sort {$a->[2] <=> $b->[2]} @_;
    my $area = green_theorem (@P);
    my $vol_bot = $area * $P[0]->[2];
    my $vol_mid = ($area * average_3d ($P[0], $P[0], $P[1])->[2]) - $vol_bot;
    my $vol_top = ($area * average_3d (@P)->[2]) - ($vol_mid + $vol_bot);
    my $cog_bot = average_3d (@P);
    $cog_bot->[2] = $P[0]->[2] /2;
    my $cog_mid = average_3d ($P[0], $P[1], [$P[1]->[0],$P[1]->[1],$P[0]->[2]],
                                             [$P[2]->[0],$P[2]->[1],$P[0]->[2]]);
    my $cog_top = average_3d ($P[0], $P[1], $P[2], [$P[2]->[0],$P[2]->[1],$P[0]->[2]]);
    return average_3d (@P) if ($vol_bot+$vol_mid+$vol_top == 0);
    scale_3d (add_3d (scale_3d ($vol_bot, $cog_bot),
                        scale_3d ($vol_mid, $cog_mid),
                        scale_3d ($vol_top, $cog_top)), 1/($vol_bot+$vol_mid+$vol_top));
}

sub average_3d
{
    return scale_3d (add_3d (@_), 1/@_);
}

sub add_3d
{
    my @coors = @_;
    my $new = [@{shift @coors}];
    for my $coor (@coors)
    {
        $new->[0] += $coor->[0];
        $new->[1] += $coor->[1];
        $new->[2] += $coor->[2];
    }
    return [$new->[0], $new->[1], $new->[2]];
}

sub rotate_3d
{
    my ($angle_rad, $a) = @_;
    ($angle_rad, $a) = ($a, $angle_rad) if ref $angle_rad;
    my $distance = distance_3d ([0,0,0], [$a->[0], $a->[1], 0]);
    my $angle;
    if ($a->[0] == 0 and $a->[1] >= 0)
    {
        $angle = 0.5 * PI();
    }
    elsif ($a->[0] == 0 and $a->[1] < 0)
    {
        $angle = 1.5 * PI();
    }
    else
    {
        $angle = atan ($a->[1]/$a->[0]);
        $angle += PI() if $a->[0] < 0;
    }
    $angle_rad += $angle;
    return [$distance * cos ($angle_rad), $distance * sin ($angle_rad), $a->[2]];
}

sub scale_3d
{
    my ($factor, $a) = @_;
    ($factor, $a) = ($a, $factor) if ref $factor;
    return [$a->[0] * $factor, $a->[1] * $factor, $a->[2] * $factor];
}

# area of a 3d triangle, always >= 0.0

sub area_3d
{
    my ($a, $b, $c) = @_;

    my $A = distance_3d ($b, $c);
    my $B = distance_3d ($a, $c);
    my $C = distance_3d ($a, $b);
    my $S = ($A + $B + $C) / 2;
    my $foo = $S * ($S - $A) * ($S - $B) * ($S - $C);
    if ($foo < 0.0)
    {
        warn "sqrt ($foo)";
        return 0.0;
    }

    return sqrt $foo;
}

# plan area of a 2d or 3d triangle.
# anti-clockwise triangles have +ve areas.
# clockwise triangles have -ve areas.

sub green_theorem
{
   my $triangle = [@_];
   my $asum = 0.0;
   for my $i (0 .. 2)
   {
      my $i1 = ($i + 1) % 3;
      my $x = $triangle->[$i]->[0];
      my $y = $triangle->[$i]->[1];
      my $x1 = $triangle->[$i1]->[0];
      my $y1 = $triangle->[$i1]->[1];
      $asum += $x*$y1 - $x1*$y;
   }
   return $asum /2;
}

sub distance_3d
{
    my ($a, $b) = @_;
    sqrt (($a->[0] - $b->[0])**2
        + ($a->[1] - $b->[1])**2
        + ($a->[2] - $b->[2])**2);
}

sub x_product_3d
{
    my ($A, $B) = @_;
    my $x = $A->[1] * $B->[2] - $B->[1] * $A->[2];
    my $y = $A->[2] * $B->[0] - $B->[2] * $A->[0];
    my $z = $A->[0] * $B->[1] - $B->[0] * $A->[1];
    return normalise_3d ([$x, $y, $z]);
}

sub dot_product_3d
{
    my ($a, $b) = @_;
    return (($a->[0] * $b->[0]) + ($a->[1] * $b->[1]) + ($a->[2] * $b->[2]));
}

sub normalise_3d
{
    my $vector = shift;
    my $magnitude = magnitude_3d ($vector);
    return [1,0,0] unless $magnitude;
    return scale_3d ($vector, 1 / $magnitude);
}

sub magnitude_3d
{
    my $vector = shift;
    return distance_3d ([0,0,0], $vector);
}

sub subtract_3d
{
    my $a = shift;
    my $b = shift;
    return [$a->[0] - $b->[0], $a->[1] - $b->[1], $a->[2] - $b->[2]];
}

sub angle_vectors_3d
{
    my ($A, $B) = @_;

    my $a = [@{$A}];
    my $b = [@{$B}];

    $a = normalise_3d ($a);
    $b = normalise_3d ($b);

    acos (($a->[0] * $b->[0]) + ($a->[1] * $b->[1]) + ($a->[2] * $b->[2]));
}

# take three sides of a triangle a_dis, b_dis,c_dis find the positive angle_A
# opposite a_dis

sub angle_triangle
{
    my ($a_dis, $b_dis, $c_dis) = @_;

    my $foo = (2 * $b_dis * $c_dis);

    if ($foo eq 0)
    {
        return 0;
    }
    elsif ($a_dis eq $b_dis + $c_dis)
    {
        return PI();
    }
    else
    {
        my $bar = (($b_dis)**2 + ($c_dis)**2 - ($a_dis)**2) / $foo;
        my $baz = (0 - $bar) * $bar + 1;
        $baz = 0.0000000000001 if $baz <= 0.0;
        return atan ((0 - $bar) / sqrt ( $baz )) + (PI() / 2);
    }
}

sub triangle_normal
{
    my ($a, $b, $c) = @_;
    return x_product_3d (subtract_3d ($b, $a), subtract_3d ($c, $a));
}

# returns a ratio between two points that intersects a plane defeined by three
# points http://paulbourke.net/geometry/planeline/

sub line_plane_intersection
{
    my ($p1, $p2, $a, $b, $c) = @_;
    my $N = triangle_normal ($a, $b, $c);
    my $top = dot_product_3d ($N, subtract_3d ($a, $p1));
    my $bottom = dot_product_3d ($N, subtract_3d ($p2, $p1));
    return ($top/$bottom) unless $bottom == 0;
    return 0.5;
}

sub point_is_in_triangle
{
    my ($A, $B, $C, $P, $threshold) = @_;
    $threshold = 0.99 unless $threshold;
    my $PA = subtract_3d ($A, $P);
    my $PB = subtract_3d ($B, $P);
    my $PC = subtract_3d ($C, $P);
    my $angle_a = angle_vectors_3d ($PB, $PC);
    my $angle_b = angle_vectors_3d ($PC, $PA);
    my $angle_c = angle_vectors_3d ($PA, $PB);
    my $angle_all = $angle_a + $angle_b + $angle_c;
    return 1 if ($angle_all/(2 * PI()) >= $threshold);
    return 0;
}

# take an anticlockwise 3d triangle and 2d coordinates for the first two
# points, find the 2d coordinate of the third

sub flatten_triangle
{
    my ($a_3d, $b_3d, $c_3d, $a_2d, $b_2d) = @_;

    # define everything needed for unplaced point

    my $distance_A = distance_3d ($b_3d, $c_3d);
    my $distance_B = distance_3d ($c_3d, $a_3d);
    my $distance_C = distance_3d ($a_3d, $b_3d);

    print STDERR join ' ', 'Distances ABC:', $distance_A, $distance_B, $distance_C, "\n"
        if $ENV{DEBUG};

    # find angle_a from a_dis, b_dis and c_dis
    # this is relative to the base line, not y=constant

    #my $angle_a = _angle ($distance_A, $distance_B, $distance_C);
    my $angle_a = angle_triangle ($distance_A, $distance_B, $distance_C);

    print STDERR join ' ', 'Angle A:', $angle_a, "\n" if $ENV{DEBUG};

    my $adjacent = ($b_2d->[0] - $a_2d->[0]) || 0.00000000001;

    my $base_angle = atan (($b_2d->[1] - $a_2d->[1]) / $adjacent);

    # correct the arctangent

    $base_angle += PI() if ($b_2d->[0] < $a_2d->[0]);

    # find the angle to c relative to y=constant
    my $new_angle = $angle_a + $base_angle;

    my $c_2d = undef;

    $c_2d->[0] = $a_2d->[0] + ($distance_B * cos ($new_angle));
    $c_2d->[1] = $a_2d->[1] + ($distance_B * sin ($new_angle));
    $c_2d->[2] = 0.0;

    return $c_2d;
}

sub PI
{
    atan2 (0,-1);
}

1;
