package File::DXF::Util;

=head1 NAME

File::DXF::Util - Useful stuff for handling AutoCAD DXF files

=head1 SYNOPSIS

Decode AutoCAD funny escaped text.

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Encode;
use Exporter;
use vars qw /@ISA @EXPORT_OK/;

@ISA = qw /Exporter/;

@EXPORT_OK = qw /decode_AutoCAD read_DXF dxf/;

=pod

Take some AutoCAD text and convert all the funny escaped characters into
Unicode:

    $text = decode_AutoCAD ($text);

=cut

sub decode_AutoCAD
{
    my $text = shift;

=pod

%%nnn Draws character number nnn in CP1252 (ie. %%128 is the euro symbol).

=cut

    $text =~ s/%%([0-9]{1,3})/decode ('CP1252', chr(${1}))/ge;

=pod

%%d Draws degrees symbol.

=cut

    $text =~ s/%%d/\x{00B0}/gi;

=pod

%%p Draws plus/minus tolerance symbol.

=cut

    $text =~ s/%%p/\x{00B1}/gi;

=pod

%%c Draws circle diameter dimensioning symbol.

Note that we are using "Latin small letter O with stroke" rather than the
preferred unicode diameter symbol.

=cut

    $text =~ s/%%c/\x{00F8}/gi;

=pod

%%% Draws a single percent sign (%).

=cut

    $text =~ s/%%%/%/g;

=pod

\U+00A9 Draws a unicode character.

=cut

    $text =~ s/\\U\+([0-9A-F]{4})/chr (hex (${1}))/ge;

    return $text;
}

=pod

Optionally, get the contents of a DXF file as a two column table suitable as
input for File::DXF::Process():

  $data = read_DXF ('/path/to/file.dxf', 'CP1252');

Note that this assumes that text in the DXF file has Windows CP1252 encoding,
which is likely if you are using AutoCAD vector fonts.  If you are using
TrueType fonts, then your text may be UTF-8 encoded.

=cut

sub read_DXF
{
    my $filename = shift;
    my $encoding = shift || 'CP1252';
    my $data = [];

    open FILE, "<:encoding($encoding)", $filename
        or die "cannot read-open $filename";
    my @raw = <FILE>;
    close FILE;

    while (@raw)
    {
        my $a = shift @raw;
        my $b = shift @raw;
        $a =~ s/(^ +| +$|\r|\n|\l)//g;
        $b =~ s/(^ +| +$|\r|\n|\l)//g;
        push @{$data}, [$a, $b];    
    }
    return $data;
}

=pod

Utility function for writing key/value data in DXF format:

  print dxf (0, 'SECTION');

=cut

sub dxf
{
    my ($key, $value) = @_;
    return " $key\n$value\n";
}

our $colour_hex = [
'#000000', '#ff0000','#ffff00','#00ff00',
'#00ffff', '#0000ff','#ff00ff','#ffffff',
'#414141', '#808080','#ff0000','#ffaaaa',
'#bd0000', '#bd7e7e','#810000','#815656',
'#680000', '#684545','#4f0000','#4f3535',
'#ff3f00', '#ffbfaa','#bd2e00','#bd8d7e',
'#811f00', '#816056','#681900','#684e45',
'#4f1300', '#4f3b35','#ff7f00','#ffd4aa',
'#bd5e00', '#bd9d7e','#814000','#816b56',
'#683400', '#685645','#4f2700','#4f4235',
'#ffbf00', '#ffeaaa','#bd8d00','#bdad7e',
'#816000', '#817656','#684e00','#685f45',
'#4f3b00', '#4f4935','#ffff00','#ffffaa',
'#bdbd00', '#bdbd7e','#818100','#818156',
'#686800', '#686845','#4f4f00','#4f4f35',
'#bfff00', '#eaffaa','#8dbd00','#adbd7e',
'#608100', '#768156','#4e6800','#5f6845',
'#3b4f00', '#494f35','#7fff00','#d4ffaa',
'#5ebd00', '#9dbd7e','#408100','#6b8156',
'#346800', '#566845','#274f00','#424f35',
'#3fff00', '#bfffaa','#2ebd00','#8dbd7e',
'#1f8100', '#608156','#196800','#4e6845',
'#134f00', '#3b4f35','#00ff00','#aaffaa',
'#00bd00', '#7ebd7e','#008100','#568156',
'#006800', '#456845','#004f00','#354f35',
'#00ff3f', '#aaffbf','#00bd2e','#7ebd8d',
'#00811f', '#568160','#006819','#45684e',
'#004f13', '#354f3b','#00ff7f','#aaffd4',
'#00bd5e', '#7ebd9d','#008140','#56816b',
'#006834', '#456856','#004f27','#354f42',
'#00ffbf', '#aaffea','#00bd8d','#7ebdad',
'#008160', '#568176','#00684e','#45685f',
'#004f3b', '#354f49','#00ffff','#aaffff',
'#00bdbd', '#7ebdbd','#008181','#568181',
'#006868', '#456868','#004f4f','#354f4f',
'#00bfff', '#aaeaff','#008dbd','#7eadbd',
'#006081', '#567681','#004e68','#455f68',
'#003b4f', '#35494f','#007fff','#aad4ff',
'#005ebd', '#7e9dbd','#004081','#566b81',
'#003468', '#455668','#00274f','#35424f',
'#003fff', '#aabfff','#002ebd','#7e8dbd',
'#001f81', '#566081','#001968','#454e68',
'#00134f', '#353b4f','#0000ff','#aaaaff',
'#0000bd', '#7e7ebd','#000081','#565681',
'#000068', '#454568','#00004f','#35354f',
'#3f00ff', '#bfaaff','#2e00bd','#8d7ebd',
'#1f0081', '#605681','#190068','#4e4568',
'#13004f', '#3b354f','#7f00ff','#d4aaff',
'#5e00bd', '#9d7ebd','#400081','#6b5681',
'#340068', '#564568','#27004f','#42354f',
'#bf00ff', '#eaaaff','#8d00bd','#ad7ebd',
'#600081', '#765681','#4e0068','#5f4568',
'#3b004f', '#49354f','#ff00ff','#ffaaff',
'#bd00bd', '#bd7ebd','#810081','#815681',
'#680068', '#684568','#4f004f','#4f354f',
'#ff00bf', '#ffaaea','#bd008d','#bd7ead',
'#810060', '#815676','#68004e','#68455f',
'#4f003b', '#4f3549','#ff007f','#ffaad4',
'#bd005e', '#bd7e9d','#810040','#81566b',
'#680034', '#684556','#4f0027','#4f3542',
'#ff003f', '#ffaabf','#bd002e','#bd7e8d',
'#81001f', '#815660','#680019','#68454e',
'#4f0013', '#4f353b','#333333','#505050',
'#696969', '#828282','#bebebe','#ffffff'];

our $colourdecimal = [
'rgb(0,0,0)', 'rgb(255,0,0)', 'rgb(255,255,0)', 'rgb(0,255,0)',
'rgb(0,255,255)', 'rgb(0,0,255)', 'rgb(255,0,255)', 'rgb(255,255,255)',
'rgb(65,65,65)', 'rgb(128,128,128)', 'rgb(255,0,0)', 'rgb(255,170,170)',
'rgb(189,0,0)', 'rgb(189,126,126)', 'rgb(129,0,0)', 'rgb(129,86,86)',
'rgb(104,0,0)', 'rgb(104,69,69)', 'rgb(79,0,0)', 'rgb(79,53,53)',
'rgb(255,63,0)', 'rgb(255,191,170)', 'rgb(189,46,0)', 'rgb(189,141,126)',
'rgb(129,31,0)', 'rgb(129,96,86)', 'rgb(104,25,0)', 'rgb(104,78,69)',
'rgb(79,19,0)', 'rgb(79,59,53)', 'rgb(255,127,0)', 'rgb(255,212,170)',
'rgb(189,94,0)', 'rgb(189,157,126)', 'rgb(129,64,0)', 'rgb(129,107,86)',
'rgb(104,52,0)', 'rgb(104,86,69)', 'rgb(79,39,0)', 'rgb(79,66,53)',
'rgb(255,191,0)', 'rgb(255,234,170)', 'rgb(189,141,0)', 'rgb(189,173,126)',
'rgb(129,96,0)', 'rgb(129,118,86)', 'rgb(104,78,0)', 'rgb(104,95,69)',
'rgb(79,59,0)', 'rgb(79,73,53)', 'rgb(255,255,0)', 'rgb(255,255,170)',
'rgb(189,189,0)', 'rgb(189,189,126)', 'rgb(129,129,0)', 'rgb(129,129,86)',
'rgb(104,104,0)', 'rgb(104,104,69)', 'rgb(79,79,0)', 'rgb(79,79,53)',
'rgb(191,255,0)', 'rgb(234,255,170)', 'rgb(141,189,0)', 'rgb(173,189,126)',
'rgb(96,129,0)', 'rgb(118,129,86)', 'rgb(78,104,0)', 'rgb(95,104,69)',
'rgb(59,79,0)', 'rgb(73,79,53)', 'rgb(127,255,0)', 'rgb(212,255,170)',
'rgb(94,189,0)', 'rgb(157,189,126)', 'rgb(64,129,0)', 'rgb(107,129,86)',
'rgb(52,104,0)', 'rgb(86,104,69)', 'rgb(39,79,0)', 'rgb(66,79,53)',
'rgb(63,255,0)', 'rgb(191,255,170)', 'rgb(46,189,0)', 'rgb(141,189,126)',
'rgb(31,129,0)', 'rgb(96,129,86)', 'rgb(25,104,0)', 'rgb(78,104,69)',
'rgb(19,79,0)', 'rgb(59,79,53)', 'rgb(0,255,0)', 'rgb(170,255,170)',
'rgb(0,189,0)', 'rgb(126,189,126)', 'rgb(0,129,0)', 'rgb(86,129,86)',
'rgb(0,104,0)', 'rgb(69,104,69)', 'rgb(0,79,0)', 'rgb(53,79,53)',
'rgb(0,255,63)', 'rgb(170,255,191)', 'rgb(0,189,46)', 'rgb(126,189,141)',
'rgb(0,129,31)', 'rgb(86,129,96)', 'rgb(0,104,25)', 'rgb(69,104,78)',
'rgb(0,79,19)', 'rgb(53,79,59)', 'rgb(0,255,127)', 'rgb(170,255,212)',
'rgb(0,189,94)', 'rgb(126,189,157)', 'rgb(0,129,64)', 'rgb(86,129,107)',
'rgb(0,104,52)', 'rgb(69,104,86)', 'rgb(0,79,39)', 'rgb(53,79,66)',
'rgb(0,255,191)', 'rgb(170,255,234)', 'rgb(0,189,141)', 'rgb(126,189,173)',
'rgb(0,129,96)', 'rgb(86,129,118)', 'rgb(0,104,78)', 'rgb(69,104,95)',
'rgb(0,79,59)', 'rgb(53,79,73)', 'rgb(0,255,255)', 'rgb(170,255,255)',
'rgb(0,189,189)', 'rgb(126,189,189)', 'rgb(0,129,129)', 'rgb(86,129,129)',
'rgb(0,104,104)', 'rgb(69,104,104)', 'rgb(0,79,79)', 'rgb(53,79,79)',
'rgb(0,191,255)', 'rgb(170,234,255)', 'rgb(0,141,189)', 'rgb(126,173,189)',
'rgb(0,96,129)', 'rgb(86,118,129)', 'rgb(0,78,104)', 'rgb(69,95,104)',
'rgb(0,59,79)', 'rgb(53,73,79)', 'rgb(0,127,255)', 'rgb(170,212,255)',
'rgb(0,94,189)', 'rgb(126,157,189)', 'rgb(0,64,129)', 'rgb(86,107,129)',
'rgb(0,52,104)', 'rgb(69,86,104)', 'rgb(0,39,79)', 'rgb(53,66,79)',
'rgb(0,63,255)', 'rgb(170,191,255)', 'rgb(0,46,189)', 'rgb(126,141,189)',
'rgb(0,31,129)', 'rgb(86,96,129)', 'rgb(0,25,104)', 'rgb(69,78,104)',
'rgb(0,19,79)', 'rgb(53,59,79)', 'rgb(0,0,255)', 'rgb(170,170,255)',
'rgb(0,0,189)', 'rgb(126,126,189)', 'rgb(0,0,129)', 'rgb(86,86,129)',
'rgb(0,0,104)', 'rgb(69,69,104)', 'rgb(0,0,79)', 'rgb(53,53,79)',
'rgb(63,0,255)', 'rgb(191,170,255)', 'rgb(46,0,189)', 'rgb(141,126,189)',
'rgb(31,0,129)', 'rgb(96,86,129)', 'rgb(25,0,104)', 'rgb(78,69,104)',
'rgb(19,0,79)', 'rgb(59,53,79)', 'rgb(127,0,255)', 'rgb(212,170,255)',
'rgb(94,0,189)', 'rgb(157,126,189)', 'rgb(64,0,129)', 'rgb(107,86,129)',
'rgb(52,0,104)', 'rgb(86,69,104)', 'rgb(39,0,79)', 'rgb(66,53,79)',
'rgb(191,0,255)', 'rgb(234,170,255)', 'rgb(141,0,189)', 'rgb(173,126,189)',
'rgb(96,0,129)', 'rgb(118,86,129)', 'rgb(78,0,104)', 'rgb(95,69,104)',
'rgb(59,0,79)', 'rgb(73,53,79)', 'rgb(255,0,255)', 'rgb(255,170,255)',
'rgb(189,0,189)', 'rgb(189,126,189)', 'rgb(129,0,129)', 'rgb(129,86,129)',
'rgb(104,0,104)', 'rgb(104,69,104)', 'rgb(79,0,79)', 'rgb(79,53,79)',
'rgb(255,0,191)', 'rgb(255,170,234)', 'rgb(189,0,141)', 'rgb(189,126,173)',
'rgb(129,0,96)', 'rgb(129,86,118)', 'rgb(104,0,78)', 'rgb(104,69,95)',
'rgb(79,0,59)', 'rgb(79,53,73)', 'rgb(255,0,127)', 'rgb(255,170,212)',
'rgb(189,0,94)', 'rgb(189,126,157)', 'rgb(129,0,64)', 'rgb(129,86,107)',
'rgb(104,0,52)', 'rgb(104,69,86)', 'rgb(79,0,39)', 'rgb(79,53,66)',
'rgb(255,0,63)', 'rgb(255,170,191)', 'rgb(189,0,46)', 'rgb(189,126,141)',
'rgb(129,0,31)', 'rgb(129,86,96)', 'rgb(104,0,25)', 'rgb(104,69,78)',
'rgb(79,0,19)', 'rgb(79,53,59)', 'rgb(51,51,51)', 'rgb(80,80,80)',
'rgb(105,105,105)', 'rgb(130,130,130)', 'rgb(190,190,190)', 'rgb(255,255,255)'];

1;
